#include <Servo.h>
#include <Wire.h>
#include "Adafruit_TCS34725.h"

// Servo motor
Servo myServo; 
const int servoPin = 9; // Connect the servo to digital pin 9

// Color sensors
Adafruit_TCS34725 tcs[] = {
    Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_614MS, TCS34725_GAIN_1X),
    Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_614MS, TCS34725_GAIN_1X),
    Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_614MS, TCS34725_GAIN_1X)
};
byte gammatable[256];

// Define constants
#define MUX_ADDR 0x70 // Multiplexer address
#define SENSORS_COUNT 3 // Number of sensors connected to the multiplexer

#define RED_PIN1 2
#define GREEN_PIN1 3
#define BLUE_PIN1 4

#define RED_PIN2 5
#define GREEN_PIN2 6
#define BLUE_PIN2 7

#define RED_PIN3 10
#define GREEN_PIN3 11
#define BLUE_PIN3 12

// Color thresholds
#define RED_THRESHOLD   100
#define GREEN_THRESHOLD 100
#define BLUE_THRESHOLD  100

// Player colors
int playerColors[3] = {0, 0, 0}; // Array to store colors for each player

// Servo movement variables
int currentPosition = 90; // Start at mid-point
int targetPosition = 90;
unsigned long lastServoMoveTime = 0;
const int servoMoveInterval = 10; // Decrease milliseconds between servo moves (was 20)
const int servoStepSize = 5; // Increase the step size

void setup() {
  Serial.begin(9600);
  Serial.println("start setup");

  // Initialize RGB LED pins
  for (int i = 2; i <= 11; i++) {
    pinMode(i, OUTPUT);
    digitalWrite(i, LOW); // Ensure all LEDs are off at start
  }
  
  Serial.println("setup leds");

  Wire.begin();    
  for (int i = 0; i < 256; i++) {
      float x = i;
      x /= 255;
      x = pow(x, 2.5);
      x *= 255;
      gammatable[i] = x;
  }
  
  Serial.println("setup 1");
  initColorSensors();
  
  Serial.println("setup after initialisation color sensors");

  // Initialize the servo motor
  myServo.attach(servoPin);
  myServo.write(currentPosition); // Move servo to start position
  Serial.println("Servo attached and set to 90 degrees");
}

void loop() 
{
  for (int i = 0; i < SENSORS_COUNT; i++) { 
    readColors(i);
  }

  // Toggle LEDs based on player colors
  for (int i = 0; i < SENSORS_COUNT; i++) {
    int redPin, greenPin, bluePin;
    switch (i) {
      case 0:
        redPin = RED_PIN1; greenPin = GREEN_PIN1; bluePin = BLUE_PIN1;
        break;
      case 1:
        redPin = RED_PIN2; greenPin = GREEN_PIN2; bluePin = BLUE_PIN2;
        break;
      case 2:
        redPin = RED_PIN3; greenPin = GREEN_PIN3; bluePin = BLUE_PIN3;
        break;
    }
    toggleRGB(redPin, greenPin, bluePin, playerColors[i]);
  }

  moveRandom(); // Move the servo motor randomly

  delay(1); // Adjust delay as needed
}

void initColorSensors() {
  Serial.println("Sensor initialization");
  for (int i = 0; i < SENSORS_COUNT; i++) { 
    chooseBus(i);
    if (tcs[i].begin()) {
      Serial.print("Found sensor "); Serial.println(i + 1);
    } else {
      Serial.println("No Sensor Found");
      while (true);
    }
  }
}

void readColors(byte sensorNum) {
  chooseBus(sensorNum);
  uint16_t r, g, b, c;
  tcs[sensorNum].getRawData(&r, &g, &b, &c);
  
  Serial.print("Sensor "); Serial.print(sensorNum); Serial.println(":");
  Serial.print("R: "); Serial.print(r, DEC); Serial.print(" ");
  Serial.print("G: "); Serial.print(g, DEC); Serial.print(" ");
  Serial.print("B: "); Serial.print(b, DEC); Serial.print(" ");
  Serial.print("Clear: "); Serial.println(c);
  
  checkColorThreshold(r, g, b, c, sensorNum);
}

void chooseBus(uint8_t bus) {
  Wire.beginTransmission(MUX_ADDR);
  Wire.write(1 << bus);
  Wire.endTransmission();
}

void checkColorThreshold(uint16_t r, uint16_t g, uint16_t b, uint16_t c, int sensorNum) {
  if (r > RED_THRESHOLD && r > g && r > b) {
    Serial.print(sensorNum);
    Serial.println(": Red detected.");
    setPlayerColor(1, sensorNum);
  } else if (g > GREEN_THRESHOLD && g > r && g > b) {
    Serial.print(sensorNum);
    Serial.println(": Green detected.");
    setPlayerColor(2, sensorNum);
  } else if (b > BLUE_THRESHOLD && b > r && b > g) {
    Serial.print(sensorNum);
    Serial.println(": Blue detected.");
    setPlayerColor(3, sensorNum);
  }
}

void setPlayerColor(int color, int sensorNum) {
  if (sensorNum >= 0 && sensorNum < SENSORS_COUNT) {
    playerColors[sensorNum] = color;
  }
}

void toggleRGB(int redPin, int greenPin, int bluePin, int color) {
  // Turn off all LEDs
  digitalWrite(redPin, LOW);
  digitalWrite(greenPin, LOW);
  digitalWrite(bluePin, LOW);

  // Depending on the color value passed, turn on the corresponding color for the specified LED
  switch (color) {
    case 1: 
      Serial.println("Turning on RED");
      digitalWrite(redPin, HIGH); 
      break;
    case 2: 
      Serial.println("Turning on GREEN");
      digitalWrite(greenPin, HIGH); 
      break;
    case 3: 
      Serial.println("Turning on BLUE");
      digitalWrite(bluePin, HIGH); 
      break;
  }
}

void moveRandom() {
  unsigned long currentTime = millis();
  if (currentTime - lastServoMoveTime >= servoMoveInterval) {
    if (currentPosition == targetPosition) {
      // Choose a new random target position
      targetPosition = random(0, 180);
      Serial.print("New target position: ");
      Serial.println(targetPosition);
    } else {
      // Move the servo multiple steps towards the target position
      int step = min(servoStepSize, abs(targetPosition - currentPosition));
      currentPosition += (targetPosition > currentPosition) ? step : -step;
      myServo.write(currentPosition);
      Serial.print("Current position: ");
      Serial.println(currentPosition);
    }
    lastServoMoveTime = currentTime;
  }
}